package jpcap.packet;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;


public class SCTPPacket extends IPPacket {
  private static final long serialVersionUID = 2427280926095446351L;

  private static final int SCTP_PROTOCOL_ID = 0x0084;
  
  /** Source port number */
  public int src_port;
  /** Destination port number */
  public int dst_port;
  /** Verification tag */
  public int ver_tag;
  /** Checksum */
  public int checksum;
  /** Chunks */
  public Chunk[] chunks;

  public SCTPPacket(IPPacket packet) throws IOException {
    /*
     * Copy Packet data
     */
    this.sec = packet.sec;
    this.usec = packet.usec;
    this.caplen = packet.caplen;
    this.len = packet.len;
    this.datalink = packet.datalink;
    this.header = packet.header;
    this.data = packet.data;
    
    /*
     * Copy IP data
     */   
    this.version = packet.version;
    this.priority = packet.priority;
    this.d_flag = packet.d_flag;
    this.t_flag = packet.t_flag;
    this.r_flag = packet.r_flag;
    this.rsv_tos = packet.rsv_tos;
    this.length = packet.length;
    this.rsv_frag = packet.rsv_frag;
    this.dont_frag = packet.dont_frag;
    this.more_frag = packet.more_frag;
    this.offset = packet.offset;
    this.hop_limit = packet.hop_limit;
    this.protocol = packet.protocol;
    this.ident = packet.ident;
    this.flow_label = packet.flow_label;
    this.src_ip = packet.src_ip;
    this.dst_ip = packet.dst_ip;
    this.option = packet.option;
    this.options = packet.options;
    
    /*
     * Parse SCTP data
     */

    InputStream is = new ByteArrayInputStream(packet.data);
    DataInputStream dis = new DataInputStream(is);

    this.src_port = dis.readShort();
    this.dst_port = dis.readShort();

    this.ver_tag = dis.readInt();

    this.checksum = dis.readInt();

    List<Chunk> chunks = new LinkedList<Chunk>();
    while (dis.available() > 0) {
      Chunk chunk = readChunk(dis);
      chunks.add(chunk);
    }
    this.chunks = chunks.toArray(new Chunk[chunks.size()]);
  }

  public static boolean isSTCPPacket(Packet packet) {
    return (packet instanceof IPPacket) && ((IPPacket) packet).protocol == SCTP_PROTOCOL_ID;
  }

  private Chunk readChunk(DataInputStream dis) throws IOException {
    Chunk chunk = new Chunk();
    chunk.type = dis.readByte();
    chunk.flags = dis.readByte();
    chunk.length = dis.readShort();

    chunk.data = new byte[chunk.length - 4];
    dis.read(chunk.data);

    // skip padding
    for (int i = 0; i < (4 - chunk.length % 4) % 4; i++)
      if (dis.available() > 0)
        dis.readByte();

    if (chunk.data.length > 0 && chunk.type == 0) {// DATA
      return new DataChunk(chunk);
    } else {
      return chunk;
    }
  }

  public static class Chunk {
    public byte type;
    public byte flags;
    public short length;

    public byte[] data;
  }

  public static class DataChunk extends Chunk {
    public int tsn;
    public short stream_id;
    public short streaq_seq_nr;
    public int protocol_id;

    public byte[] payload;

    public DataChunk(Chunk chunk) throws IOException {
      this.type = 0;
      this.flags = chunk.flags;
      this.length = chunk.length;
      this.data = chunk.data;

      DataInputStream dis = new DataInputStream(new ByteArrayInputStream(data));
      this.tsn = dis.readInt();
      this.stream_id = dis.readShort();
      this.streaq_seq_nr = dis.readShort();
      this.protocol_id = dis.readInt();

      this.payload = new byte[data.length - 12];
      dis.read(payload);
    }
  }
}
